#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# create_QR_codes_of_public_and_private_keys                                   #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program creates a QR code for a specified public key and private key    #
# and enables optional specification of the size of the resulting PNG images.  #
# This program loads the keys from a Python file (keys.py by default) which    #
# defines the string variables key_public and key_private.                     #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help       display help message
    --version        display version and exit

    --size=INT       width and height in pixels of QR codes  [default: 602]
    --keys=FILEPATH  keys file                               [default: keys.py]
"""

import docopt
import os
import PIL.Image

import pyqrcode

name    = "create_QR_codes_of_public_and_private_keys"
version = "2018-01-09T2013Z"
logo    = None

def main(options):

    filepath_keys = options["--keys"]
    size          = int(options["--size"])

    filepath_keys = os.path.expanduser(filepath_keys)

    if os.path.isfile(filepath_keys):

        with open(filepath_keys, "r") as file_keys:
            file_string = file_keys.read()
            keys = {}
            exec(file_string, keys)
            key_public  = keys["key_public"]
            key_private = keys["key_private"]

        if key_public:
            qr = pyqrcode.create(key_public)
            qr.png("key_public.png", scale = 50, quiet_zone = 0)
            resize_image(filename = "key_public.png", size = size)

        if key_private:
            qr = pyqrcode.create(key_private)
            qr.png("key_private.png", scale = 50, quiet_zone = 0)
            resize_image(filename = "key_private.png", size = size)

    else:

        print("no keys file {filepath_keys} found".format(filepath_keys = filepath_keys))
        sys.exit()

def resize_image(
    filename = None,
    size     = 602
    ):

    image    = PIL.Image.open(filename)
    fraction = size / float(image.size[0])
    hsize    = int(float(image.size[1]) * float(fraction))
    image    = image.resize((size, hsize), PIL.Image.ANTIALIAS)
    image.save(filename) 

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# detect_transaction_RBS                                                       #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program gets the recent transactions of an RBS account using Firefox    #
# and Selenium and searches the transactions for a specified reference. It     #
# does this in a loop until the transaction is detected.                       #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help              display help message
    --version               display version and exit

    --reference=TEXT        reference text to search for   [default: none]

    --interval=INT          time between recordings (s)    [default: 120]

    --credentials=FILEPATH  credentials file               [default: ~/.rbs]

    --sleep_time=INT        time between web interactions  [default: 1]
"""

import docopt
import time
import sys

import RBS

name    = "detect_transaction_RBS"
version = "2017-11-28T1851Z"
logo    = None

def main(options):

    reference            =     options["--reference"]
    interval             = int(options["--interval"])
    filepath_credentials =     options["--credentials"]
    sleep_time           = int(options["--sleep_time"])

    if reference == "none":
        print("reference not specified")
        sys.exit()

    found = False

    while not found:

        try:
            status = RBS.account_status(
                filepath_credentials = filepath_credentials,
                sleep_time           = sleep_time
            )
            df = status["transactions"]
            if df[df["description"].str.contains(reference)].values.any():
                print("transaction found:\n\n" + str(df[df["description"].str.contains(reference)]))
                found = True
            else:
                print("\ntransaction not found -- current transactions:\n")
                print(df)
                print("\nwait " + str(interval) + " s")
        except:
            print("error getting transactions")
            pass
        if not found:
            time.sleep(interval)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# banks                                                                        #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program provides banking and other utilities.                           #
#                                                                              #
# copyright (C) 2018 William Breaden Madden, Liam Moore                        #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

import os
import requests
import sys

import pandas as pd
import pyprel
from pymonzo import MonzoAPI

name    = "banks"
version = "2018-02-07T0053Z"

def bank_access_ok(
    bank = None
    ):
    """
    Check bank API access.
    """
    try:
        if bank == "Monzo":     df = transactions_DataFrame_Monzo()
        if bank == "RBS":       df = transactions_DataFrame_RBS()
        if bank == "Santander": df = transactions_DataFrame_Santander()
        if isinstance(df, pd.DataFrame):
            return True
        else:
            return False
    except:
        print("error -- bank access ok")
        return False

def transactions_DataFrame_Monzo(
    print_table          = False
    ):
    """
    Return a DataFrame of bank account transactions.
    """
    # API
    try:
        monzo = MonzoAPI()
        transactions = [transaction._raw_data for transaction in monzo.transactions()]
    except:
        print("error accessing Monzo API")
        return False
    # DataFrame
    df = pd.DataFrame(columns = [
        "datetime",
        "amount",
        "description",
        "notes",
        "counterparty_reference",
        "counterparty_account_number",
        "counterparty_sort_code",
        "counterparty_name",
        "currency",
        "category",
        "scheme",
        "id"
    ])
    for transaction in transactions:
        try:
            counterparty_account_number = transaction["counterparty"]["account_number"]
            counterparty_sort_code      = transaction["counterparty"]["sort_code"]
            counterparty_name           = transaction["counterparty"]["name"]
            counterparty_reference      = transaction["description"]
        except:
            counterparty_account_number = "-"
            counterparty_sort_code      = "-"
            counterparty_name           = "-"
            counterparty_reference      = "-"
        try:
            try:
                counterparty_account_number = transaction["counterparty"]["account_number"]
                counterparty_sort_code      = transaction["counterparty"]["sort_code"]
                counterparty_name           = transaction["counterparty"]["name"]
                counterparty_reference      = transaction["description"]
            except:
                counterparty_account_number = "-"
                counterparty_sort_code      = "-"
                counterparty_name           = "-"
                counterparty_reference      = "-"
            df = df.append(
                {
                    "datetime"                   : pd.to_datetime(transaction["created"]).to_pydatetime(),
                    "amount"                     : float(transaction["amount"]) / float(100),
                    "description"                : transaction["description"],
                    "notes"                      : transaction["notes"],
                    "counterparty_reference"     : counterparty_reference,
                    "counterparty_account_number": counterparty_account_number,
                    "counterparty_sort_code"     : counterparty_sort_code,
                    "counterparty_name"          : counterparty_name,
                    "currency"                   : transaction["currency"],
                    "category"                   : transaction["category"],
                    "scheme"                     : transaction["scheme"],
                    "id"                         : transaction["id"]
                },
                ignore_index = True
            )
        except:
            print("error -- transactions DataFrame Monzo")
            pass
    if print_table:
        print(table_transactions(df = df))
    return df

def transactions_DataFrame_RBS(
    filepath_credentials = "~/.rbs",
    print_table          = False
    ):
    """
    Return a DataFrame of bank account transactions.
    """
    # credentials
    filepath_credentials = os.path.expanduser(filepath_credentials)
    if not os.path.isfile(filepath_credentials):
        print("no credentials file {filepath} found".format(filepath = filepath_credentials))
        sys.exit()
    with open(filepath_credentials, "r") as file_credentials:
        credentials_string = file_credentials.read()
    credentials = {}
    exec(credentials_string, credentials)
    token        = credentials["token_teller"]
    account_code = credentials["account_code_teller"]
    # API
    try:
        URL      = "https://api.teller.io/accounts/" + account_code + "/transactions"
        headers  = {"Authorization": "Bearer " + token}
        data     = {"" : ""}
        response = requests.get(URL, json = data, headers = headers).json()
    except:
        print("error accessing Teller RBS API")
        return False
    # DataFrame
    df = pd.DataFrame(columns = ["date"])
    for transaction in response:
        df = df.append(
            {
                "date"                  : transaction["date"],
                "counterparty_reference": transaction["counterparty"],
                "description"           : transaction["description"],
                "id"                    : transaction["id"],
                "amount"                : float(transaction["amount"])
            },
            ignore_index = True
        )
    if print_table:
        print(table_transactions(df = df))
    return df

def transactions_DataFrame_bank(
    bank                 = None,
    filepath_credentials = "~/.rbs",
    print_table          = False
    ):
    try:
        if bank == "Monzo":
            return transactions_DataFrame_Monzo(
                print_table          = print_table
            )["valid"]
        elif bank == "RBS":
            return transactions_DataFrame_RBS(
                filepath_credentials = filepath_credentials,
                print_table          = print_table
            )["valid"]
        elif bank == "Santander":
            return transactions_DataFrame_Santander(
                print_table          = print_table
            )["valid"]
        else:
            return False
    except:
        print("error -- transactions DataFrame bank")
        return False

def payment_in_transactions(
    df          = None,
    reference   = None,
    amount      = None,
    print_table = False
    ):
    """
    Search the fields "counterparty_reference", "description" and "notes" (if it
    for a specified reference in a specified transactions DataFrame. If the
    reference is found, check the amount of the payment is correct by comparing
    the specified amount with the "amount" field.

    Return a dictionary of the following form:

    {
        "reference_found":   bool,      # True if reference found
        "amount_correct":    bool,      # True if sum of amounts found is amount specified
        "valid":             bool,      # True if reference found and amount correct
        "transaction"        DataFrame, # DataFrame of matches
        "amount_difference": float      # difference between amount specified and sum of amounts found
    }
    """
    reference = str(reference)
    amount    = float(amount)
    if "notes" in df.columns:
        matches = df[(df["counterparty_reference"].str.contains(reference)) | (df["description"].str.contains(reference)) | (df["notes"].str.contains(reference))]
    else:
        matches = df[(df["counterparty_reference"].str.contains(reference)) | (df["description"].str.contains(reference))]
    reference_found   = not matches.empty
    amount_correct    = sum(matches["amount"].values) == amount
    valid             = reference_found and amount_correct
    amount_difference = amount - sum(matches["amount"].values)
    transaction       = matches
    if print_table:
        print(table_transactions(df = matches))
    return {
        "reference_found":   reference_found,
        "amount_correct":    amount_correct,
        "valid":             valid,
        "amount_difference": amount_difference,
        "transactions":      matches
    }

def payment_in_transactions_Monzo(
    reference            = None,
    amount               = None,
    print_table          = False
    ):
    return payment_in_transactions(
        df          = transactions_DataFrame_Monzo(),
        reference   = reference,
        amount      = amount,
        print_table = print_table
    )

def payment_in_transactions_RBS(
    reference            = None,
    amount               = None,
    filepath_credentials = "~/.rbs",
    print_table          = False
    ):
    return payment_in_transactions(
        df          = transactions_DataFrame_RBS(filepath_credentials = filepath_credentials),
        reference   = reference,
        amount      = amount,
        print_table = print_table
    )

def payment_in_transactions_bank(
    reference            = None,
    amount               = None,
    bank                 = None,
    filepath_credentials = "~/.rbs",
    print_table          = False
    ):
    try:
        if bank == "Monzo":
            return payment_in_transactions_Monzo(
                reference            = reference,
                amount               = amount,
                print_table          = print_table
            )["valid"]
        elif bank == "RBS":
            return payment_in_transactions_RBS(
                reference            = reference,
                amount               = amount,
                filepath_credentials = filepath_credentials,
                print_table          = print_table
            )["valid"]
        elif bank == "Santander":
            return payment_in_transactions_Santander(
                reference            = reference,
                amount               = amount
            )["valid"]
        else:
            return False
    except:
        print("error -- payment in transactions bank")
        return False

def table_transactions(
    df = None
    ):
    return pyprel.Table(contents = pyprel.table_DataFrame(df = df))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# login_web_LocalBitcoins                                                      #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program logs into the LocalBitcoins website using Selenium.             #
#                                                                              #
# copyright (C) 2018 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help   display help message
    --version    display version and exit
"""

import docopt
import os
import time
import sys

import che_guevara_otp
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait as wait

name    = "login_web_LocalBitcoins"
version = "2018-01-31T1732Z"

def main(options):

    filepath_credentials = os.path.expanduser("~/.lbc")
    if not os.path.isfile(filepath_credentials):
        print("no credentials file {filepath} found".format(filepath = filepath_credentials))
        sys.exit()
    with open(filepath_credentials, "r") as file_credentials:
        credentials_string = file_credentials.read()
    credentials = {}
    exec(credentials_string, credentials)
    username = credentials["username"]
    passcode = credentials["passcode"]
    secret   = credentials["secret"]

    driver = webdriver.Firefox()
    driver.get("https://localbitcoins.com/accounts/login/")
    time.sleep(2)
    # username
    element = driver.find_element_by_name("username")
    element.send_keys(username)
    # passcode
    element = driver.find_element_by_name("password")
    element.send_keys(passcode)
    time.sleep(1)
    # reCAPTCHA
    wait(driver, 10).until(EC.frame_to_be_available_and_switch_to_it(driver.find_element_by_xpath('//iframe[contains(@src, "google.com/recaptcha")]')))
    wait(driver, 10).until(EC.element_to_be_clickable((By.ID, "recaptcha-anchor"))).click()
    time.sleep(2)
    # login button
    driver.switch_to.default_content()
    elements = driver.find_elements_by_xpath("//button[contains(text(), 'Login')]")
    elements[0].click()
    # OTP   
    element = driver.find_element_by_name("token")
    element.send_keys(che_guevara_otp.TOTP(secret = secret))
    element.send_keys(Keys.RETURN)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os

import setuptools

def main():

    setuptools.setup(
        name             = "denarius",
        version          = "2018.04.03.2341",
        description      = "currency and other utilities",
        long_description = long_description(),
        url              = "https://github.com/wdbm/denarius",
        author           = "Will Breaden Madden",
        author_email     = "wbm@protonmail.ch",
        license          = "GPLv3",
        packages         = [
                           "denarius"
                           ],
        install_requires = [
                           "beautifulsoup4",
                           "che_guevara_otp",
                           "currencyconverter",
                           "dataset",
                           "datavision",
                           "forex-python",
                           "graphviz",
                           "krakenex",
                           "lock",
                           "pandas",
                           "pymonzo",
                           "pypng",
                           "pyprel",
                           "pyqrcode",
                           "pytrends",
                           "propyte",
                           "selenium",
                           "sklearn"
                           ],
        scripts          = [
                           "buy_BTC_for_EUR_last_price_on_Kraken.py",
                           "create_paper_wallet.py",
                           "create_QR_codes_of_public_and_private_keys.py",
                           "denarius_graph_Bitcoin.py",
                           "denarius_loop_save_KanoPool.py",
                           "denarius_loop_save_Nanopool.py",
                           "denarius_loop_save_SlushPool.py",
                           "denarius_save_stock_prices.py",
                           "detect_transaction_RBS.py",
                           "get_account_balance_RBS.py",
                           "login_web_Kraken.py",
                           "login_web_LocalBitcoins.py",
                           "loop_display_arbitrage_data.py",
                           "loop_save_arbitrage_data_Kraken_LocalBitcoins_UK.py",
                           "loop_save_LocalBitcoins_values_to_database.py",
                           "loop_save_RBS_to_CSV.py",
                           "loop_save_Santander_to_CSV.py",
                           "print_Kraken_balances.py",
                           "print_Kraken_last_price_BTC_EUR.py",
                           "print_table_bank_account.py",
                           "web_display_arbitrage_data.py"
                           ]
    )

def long_description(
    filename = "README.md"
    ):

    if os.path.isfile(os.path.expandvars(filename)):
        try:
            import pypandoc
            long_description = pypandoc.convert_file(filename, "rst")
        except ImportError:
            long_description = open(filename).read()
    else:
        long_description = ""
    return long_description

if __name__ == "__main__":
    main()
